import os
import requests
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.shortcuts import get_object_or_404
from django.utils.timezone import now
from decouple import config
# from crea import settings
from django.conf import settings
from notifications_app.models import Notification
from src.ChatTokenBuilder2 import ChatTokenBuilder
from .models import ActiveStatus, CallSession, ChatRoom, GroupChat, GroupMessage, GroupReport, PersonalChat, MediaFile
from .serializers import ChatRoomSerializer, CreateGroupSerializer, MessageSendSerializer, SendGroupMessageSerializer
from django.contrib.auth import get_user_model
from django.db.models import Q, Max
from rest_framework.permissions import IsAuthenticated
from chat_app import models
from rest_framework import generics
from cryptography.fernet import Fernet
from rest_framework import viewsets, status
from django.core.paginator import Paginator

User = get_user_model()
ENCRYPTION_KEY = config('ENCRYPTION_KEY')
cipher = Fernet(ENCRYPTION_KEY)

APP_ID = config('APP_ID')
APP_CERTIFICATE = config('APP_CERTIFICATE')

class MessageSendAPI(APIView):
    def post(self, request):
        serializer = MessageSendSerializer(data=request.data)
        if serializer.is_valid():
            # Extract data from request
            receiver_id = serializer.validated_data['receiver_id']
            message_text = serializer.validated_data.get('message', '').strip()
            images = serializer.validated_data.get('images', [])
            video = serializer.validated_data.get('video', None)
            file = serializer.validated_data.get('file', None)
            voice = serializer.validated_data.get('voice', None)

            sender = request.user
            receiver = get_object_or_404(User, id=receiver_id)
            # chatroom = ChatRoom.objects.filter(members=sender).filter(members=receiver).first()
            chatroom = ChatRoom.objects.filter(
                Q(owner=sender) | Q(members=sender),
                Q(owner=receiver) | Q(members=receiver)
            ).first()
            # print(chatroom.conversation_id)
            print(sender.agora_username)


            encrypted_message = cipher.encrypt(message_text.encode()).decode() if message_text else None

            # Create the personal chat message
            personal_chat = PersonalChat.objects.create(
                sender=sender,
                receiver=receiver,
                content= encrypted_message,
                timestamp=now(),
            )

            # Save images as media files
            media_urls = []
            for image in images:
                media_file = MediaFile.objects.create(
                    file=image,
                    media_type='image',
                )
                if not media_file or not media_file.file:
                    return Response({"error": "File was not saved correctly"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
                print(f"Media File Created: {media_file}")  # Debugging
                print(f"File URL: {getattr(media_file.file, 'url', 'No file uploaded')}")
                personal_chat.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            # Save video as a media file (if provided)
            if video:
                media_file = MediaFile.objects.create(
                    file=video,
                    media_type='video',
                )
                personal_chat.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if file:
                media_file = MediaFile.objects.create(
                    file=file,
                    media_type='file',
                )
                personal_chat.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if voice:
                media_file = MediaFile.objects.create(
                    file=voice,
                    media_type='voice',
                )
                personal_chat.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if message_text:
                # Text message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [chatroom.conversation_id],
                    "type": "txt",
                    "body": {
                        "type": "txt",
                        "msg": message_text,
                    },
                }
            elif images:
                # Image message payload (assuming only one image is sent)
                image = images[0]
                media_file = MediaFile.objects.filter(file=image).first()

                payload = {
                    "from": sender.agora_username,
                    "to": [chatroom.conversation_id],
                    "type": "img",
                    "body": {
                        "filename": image.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 480,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            elif video:
                # Video message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [chatroom.conversation_id],
                    "type": "video",
                    "body": {
                        "filename": video.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 1280,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            elif file:
                # Video message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [chatroom.conversation_id],
                    "type": "file",
                    "body": {
                        "filename": file.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 1280,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            elif voice:
                # Video message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [chatroom.conversation_id],
                    "type": "audio",
                    "body": {
                        "filename": voice.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 1280,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            else:
                return Response({"error": "No valid message content provided"}, status=status.HTTP_400_BAD_REQUEST)

            # Call Agora API to send the message
            app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, settings.AGORA_EXPIRE_TIME)
            agora_url = f"{settings.AGORA_BASE_URL}/messages/chatrooms"
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {app_token}",
            }
            response = requests.post(agora_url, json=payload, headers=headers)
            print(response)
            if response.status_code != 200:
                # Log the error or handle it appropriately
                print("Agora API Error:", response.status_code, response.text)
                return Response({"error": "Failed to send message via Agora"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # Prepare response data
            response_data = {
                "success": True,
                "code": 200,
                "error": False,
                "data": {
                    "message_id": personal_chat.id,
                    "sender_id": sender.id,
                    "receiver_id": receiver.id,
                    "message": personal_chat.content,
                    "timestamp": personal_chat.timestamp.isoformat(),
                    "media": [
                        {
                            "file_url": media.file.url,
                            "media_type": media.media_type,
                            "file_name":media.file.name.split('/')[-1],
                        } for media in personal_chat.media_files.all()
                    ],
                    "read": personal_chat.is_read,
                }
            }
            return Response(response_data, status=status.HTTP_200_OK)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class CreateGroupAPI(APIView):
    def post(self, request):
        member_ids_str = request.POST.get('members', '')  # Default to empty string if not provided
        print(type(member_ids_str))
        profile_image = request.FILES.get('profile_image', None)
        serializer = CreateGroupSerializer(data=request.data)
        if serializer.is_valid():
            group_name = serializer.validated_data['name']
            description = serializer.validated_data.get('description', '')
            admin = request.user
            # member_ids = serializer.validated_data['members']
            # member_ids = request.POST.getlist('members'
            # member_ids = request.data.get("members", [])
            # profile_image = request.FILES.get('profile_image', None)

            try:
                member_ids = list(map(int, member_ids_str.split(','))) if member_ids_str else []
            except ValueError:
                return Response({"error": "Invalid member ID format. Expected a comma-separated list of integers."},
                                status=status.HTTP_400_BAD_REQUEST)
            

            # Ensure admin is always part of the group
            members = User.objects.filter(id__in=member_ids)
            if admin not in members:
                members = list(members) + [admin]

            # agora_usernames = [user.agora_username for user in members if hasattr(user, 'agora_username')]
            valid_members = []
            missing_users = []
            
            for user in members:
                if hasattr(user, 'agora_username') and user.agora_username:
                    valid_members.append(user)
                else:
                    missing_users.append(user.username)

            if missing_users:
                return Response(
                    {
                        "error": "Some users are not registered on Agora",
                        "missing_users": missing_users,
                    },
                    status=status.HTTP_400_BAD_REQUEST,
                )


            agora_payload = {
                "groupname": group_name,
                "desc": description,
                "public": False,  # Set to True for public groups, False for private
                "maxusers": 300,  # Default to 300 users
                "owner": admin.agora_username,
                "members": [user.agora_username for user in valid_members]
            }

            # Call Agora API to create the group
            app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, settings.AGORA_EXPIRE_TIME)
            agora_url = f"{settings.AGORA_BASE_URL}/chatgroups"
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {app_token}",  # Replace with actual token
            }
            response = requests.post(agora_url, json=agora_payload, headers=headers)

            if response.status_code != 200:
                # Log the error or handle it appropriately
                print("Agora API Error:", response.status_code, response.text)
                return Response({"error": "Failed to create group on Agora"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # Extract Agora group ID from the response
            agora_response = response.json()
            agora_group_id = agora_response.get("data", {}).get("groupid")

            if not agora_group_id:
                return Response({"error": "Failed to retrieve Agora group ID"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # Create the group
            group = GroupChat.objects.create(
                name=group_name,
                description=description,
                admin=admin,
                agora_group_id=agora_group_id,
                profile_image=profile_image 
            )
            group.members.add(*members)

            # cleaned_members = [member.agora_username.lstrip("@") for member in valid_members]
            # cleaned_owner = admin.agora_username.lstrip("@")

            # chatroom_payload = {
            #     "name": group_name,
            #     "description": description,
            #     "maxusers": 300,
            #     "owner": admin.agora_username,
            #     "members": [user.agora_username for user in valid_members],
            # }

            # chatroom_response = requests.post(
            #     settings.AGORA_BASE_URL + "/chatrooms",
            #     json=chatroom_payload,
            #     headers={"Content-Type": "application/json", "Authorization": f"Bearer {app_token}"},
            # )

            # if chatroom_response.status_code != 200:
            #     return Response({"error": "Failed to create chat room on Agora"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # chatroom_data = chatroom_response.json()
            # chatroom_id = chatroom_data.get("data", {}).get("id")

            # if not chatroom_id:
            #     return Response({"error": "Failed to retrieve Agora chat room ID"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # # Save ChatRoom in Database & Link to Group
            # chatroom = ChatRoom.objects.create(
            #     conversation_id=chatroom_id,
            #     name=group_name,
            #     description=description,
            #     owner=admin,
            #     max_users=300,
            #     group = group,
            # )
            # chatroom.members.add(*valid_members)


            return Response(
                {
                    "success": True,
                    "code": 200,
                    "data": {
                        "group_id": group.id,
                        # "chatroom_id": chatroom.id,
                        "name": group.name,
                        "description": group.description,
                        "admin": admin.username,
                        "members": [user.username for user in group.members.all()],
                        "agora_group_id": agora_group_id,
                        # "agora_chatroom_id": chatroom_id,
                    },
                },
                status=status.HTTP_200_OK,
            )
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
class GetPersonalChatsAPI(APIView):
    def get(self, request, other_user_id):
        sender = request.user
        receiver = get_object_or_404(User, id=other_user_id)

        # Filter personal chats between the two users
        chats = PersonalChat.objects.filter(
            (Q(sender=sender) & Q(receiver=receiver))
            | (Q(sender=receiver) & Q(receiver=sender))
        ).exclude(deleted_for=sender).order_by('timestamp')

        page = request.GET.get('page', 1)  # Default to page 1
        page_size = request.GET.get('page_size', 10)  # Default page size = 10
        paginator = Paginator(chats, page_size)
        paginated_chats = paginator.get_page(page)
        

        # Prepare response
        # chat_data = [
        #     {
        #         "id": chat.id,
        #         "sender_id": chat.sender.id,
        #         "sender_full_name": chat.sender.full_name,
        #         "sender_username": chat.sender.username,
        #         "sender_profile_image": (
        #             request.build_absolute_uri(chat.sender.profile.profile_image.url)
        #             if hasattr(chat.sender, "profile") and chat.sender.profile.profile_image
        #             else None
        #         ),
        #         # "receiver_id": chat.receiver.id,
        #         # "receiver_full_name": chat.receiver.full_name,
        #         # "receiver_username": chat.receiver.username,
        #         # "receiver_profile_image": (
        #         #     request.build_absolute_uri(chat.receiver.profile.profile_image.url)
        #         #     if hasattr(chat.receiver, "profile") and chat.receiver.profile.profile_image
        #         #     else None
        #         # ),
        #         "message": cipher.decrypt(chat.content.encode()).decode() if chat.content else None,
        #         "timestamp": chat.timestamp.isoformat(),
        #         "type":"",
        #         "media": [
        #             {
        #                 "file_url": media.file.url,
        #                 "media_type": media.media_type,
        #             } for media in chat.media_files.all()
        #         ],
        #         "read": chat.is_read,
        #         "user_type": "sender" if chat.sender == sender else "receiver",
        #     }
        #     for chat in chats
        # ]
        
        chat_data = []
        for chat in paginated_chats:
            # Determine message type directly from MediaFile
            if chat.content:
                message_type = "text"
            elif chat.media_files.exists():
                message_type = chat.media_files.first().media_type  # Directly get media type
            else:
                message_type = "unknown"
            sender_status = getattr(chat.sender, 'active_status_chat_app', None)
            presence_status = sender_status.status if sender_status else 3  # Default to 'Offline'

            chat_data.append({
                "id": chat.id,
                "sender_id": chat.sender.id,
                "sender_full_name": chat.sender.full_name,
                "sender_username": chat.sender.username,
                "sender_profile_image": (
                    request.build_absolute_uri(chat.sender.profile.profile_image.url)
                    if hasattr(chat.sender, "profile") and chat.sender.profile.profile_image
                    else None
                ),
                "message": cipher.decrypt(chat.content.encode()).decode() if chat.content else None,
                "timestamp": chat.timestamp.isoformat(),
                "type": message_type,  # Directly using media_type from MediaFile
                "media": [
                    {
                        "file_url": media.file.url,
                        "media_type": media.media_type,
                        "file_name":media.file.name.split('/')[-1]
                    } for media in chat.media_files.all()
                ],
                "presence_status": presence_status,
                "read": chat.is_read,
                "user_type": "sender" if chat.sender == sender else "receiver",
            })


        return Response(
            {
                "success": True,
                "code": 200,
                "data": chat_data,
                "current_page": paginated_chats.number,
                "total_pages": paginator.num_pages,
                "total_items": paginator.count,
                "has_next": paginated_chats.has_next(),
                "has_previous": paginated_chats.has_previous(),
            },
            status=status.HTTP_200_OK,
        )

class SendGroupMessageAPI(APIView):
    def post(self, request):
        serializer = SendGroupMessageSerializer(data=request.data)
        if serializer.is_valid():
            group_id = serializer.validated_data['group_id']
            content = serializer.validated_data.get('content', '').strip()
            images = serializer.validated_data.get('images', [])
            video = serializer.validated_data.get('video', None)
            file = serializer.validated_data.get('file', None)
            voice = serializer.validated_data.get('voice', None)

            # Fetch the group
            group = get_object_or_404(GroupChat, id=group_id)

            # group_chatroom = GroupChat.objects.filter(id = group_id)
            # print(group_chatroom.agora_group_id)
            print(group.agora_group_id)

            encrypted_message = cipher.encrypt(content.encode()).decode() if content else None

            # Ensure the sender is a member of the group
            sender = request.user
            if sender not in group.members.all():
                return Response(
                    {"error": "You are not a member of this group."},
                    status=status.HTTP_403_FORBIDDEN,
                )

            # Create the group message
            group_message = GroupMessage.objects.create(
                group=group,
                sender=sender,
                content=encrypted_message if content else None,
                timestamp=now(),
            )

            # Save media files as related media
            media_urls = []
            for image in images:
                media_file = MediaFile.objects.create(file=image, media_type='image')
                
                if not media_file or not media_file.file:
                    return Response({"error": "File was not saved correctly"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
                group_message.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if video:
                media_file = MediaFile.objects.create(file=video, media_type='video')
                group_message.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if file:
                media_file = MediaFile.objects.create(
                    file=file,
                    media_type='file',
                )
                group_message.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if voice:
                media_file = MediaFile.objects.create(
                    file=voice,
                    media_type='voice',
                )
                group_message.media_files.add(media_file)
                media_urls.append(media_file.file.url)

            if content:
                # Text message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [group.agora_group_id],
                    "type": "txt",
                    "body": {
                        "type": "txt",
                        "msg": content,
                    },
                }
            elif images:
                # Image message payload (assuming only one image is sent)
                image = images[0]
                media_file = MediaFile.objects.filter(file=image).first()

                payload = {
                    "from": sender.agora_username,
                    "to": [group.agora_group_id],
                    "type": "img",
                    "body": {
                        "filename": image.name,
                        "url": media_urls[0],
                    },
                }
            elif video:
                # Video message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [group.agora_group_id],
                    "type": "video",
                    "body": {
                        "filename": video.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 1280,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            elif file:
                # Video message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [group.agora_group_id],
                    "type": "file",
                    "body": {
                        "filename": file.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 1280,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            elif voice:
                # Video message payload
                payload = {
                    "from": sender.agora_username,
                    "to": [group.agora_group_id],
                    "type": "audio",
                    "body": {
                        "filename": voice.name,
                        # "secret": "your_secret_key",  # Replace with actual secret
                        "url": media_urls[0],
                        # "size": {
                        #     "width": 1280,  # Replace with actual dimensions
                        #     "height": 720,
                        # },
                    },
                }
            else:
                return Response({"error": "No valid message content provided"}, status=status.HTTP_400_BAD_REQUEST)
            
            app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, settings.AGORA_EXPIRE_TIME)
            agora_url = f"{settings.AGORA_BASE_URL}/messages/chatgroups"
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {app_token}",
            }
            response = requests.post(agora_url, json=payload, headers=headers)
            print(response)
            if response.status_code != 200:
                # Log the error or handle it appropriately
                print("Agora API Error:", response.status_code, response.text)
                return Response({"error": "Failed to send message in group via Agora"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


            # Response data
            response_data = {
                "success": True,
                "code": 200,
                "data": {
                    "group_id": group.id,
                    "group_name": group.name,
                    "message_id": group_message.id,
                    "sender_id": sender.id,
                    "sender_name": sender.username,
                    "content": group_message.content,
                    "timestamp": group_message.timestamp.isoformat(),
                    "media": [
                        {
                            "file_url": media.file.url,
                            "media_type": media.media_type,
                        }
                        for media in group_message.media_files.all()
                    ],
                },
            }
            return Response(response_data, status=status.HTTP_200_OK)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
class MarkMessagesAsRead(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, *args, **kwargs):
        """
        Mark messages as read in a personal chat when a user opens the chat.
        """
        current_user = request.user
        other_user_id = request.data.get('other_user_id')

        if not other_user_id:
            return Response(
                {"error": "The 'other_user_id' field is required."},
                status=status.HTTP_400_BAD_REQUEST
            )

        try:
            # Filter messages sent by the other user to the current user that are unread
            unread_messages = PersonalChat.objects.filter(
                sender_id=other_user_id,
                receiver_id=current_user.id,
                is_read=False
            )
            
            # Mark them as read
            unread_messages.update(is_read=True)

            return Response(
                {"message": "Messages marked as read."},
                status=status.HTTP_200_OK
            )

        except Exception as e:
            return Response(
                {"error": str(e)},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )
        
class UserActiveStatusView(APIView):
    def get(self, request):
        user = request.user
        try:
            active_status = ActiveStatus.objects.get(user=user)
            status_display = active_status.get_status_display()
        except ActiveStatus.DoesNotExist:
            return Response({
                "success": False,
                "code": 404,
                "error": "User status not found."
            }, status=status.HTTP_404_NOT_FOUND)

        return Response({
            "success": True,
            "code": 200,
            "status": status_display,
            "last_updated": active_status.updated_at
        }, status=status.HTTP_200_OK)



    def put(self, request):
        new_status = request.data.get("status")
        user = request.user

        if int(new_status) not in [1, 2, 3]:

            return Response({
                "success": False,
                "code": 400,
                "error": "Invalid status. Valid statuses are: 1 (Active), 2 (Inactive), 3 (Offline)."
            }, status=status.HTTP_400_BAD_REQUEST)

        active_status, created = ActiveStatus.objects.update_or_create(
            user=user,
            defaults={"status": int(new_status)}
        )

        return Response({
            "success": True,
            "code": 200,
            "message": "User status updated successfully.",
            "status": active_status.get_status_display(),
        }, status=status.HTTP_200_OK)
    
class ChatListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        """
        Get the list of users who exchanged messages with the current user.
        Returns the last message, unread message count, and user details.
        """
        current_user = request.user
        conversations = PersonalChat.objects.filter(
            Q(sender=current_user) | Q(receiver=current_user)
        ).values(
            "sender", "receiver"
        ).annotate(
            last_message_time=Max("timestamp"),
        ).order_by("-last_message_time")

        # Prepare data for response
        user_ids = set()
        last_message_map = {}
        unread_message_count = {}

        for chat in conversations:
            sender_id = chat["sender"]
            receiver_id = chat["receiver"]

            other_user_id = receiver_id if sender_id == current_user.id else sender_id
            user_ids.add(other_user_id)

            # Fetch the last message for the conversation
            last_message = PersonalChat.objects.filter(
                Q(sender=sender_id, receiver=receiver_id) |
                Q(sender=receiver_id, receiver=sender_id)
            ).order_by("-timestamp").first()

            if last_message:
                last_message_map[other_user_id] = {
                    "message": last_message.content,
                    "timestamp": last_message.timestamp,
                }

            # Count unread messages
            unread_messages = PersonalChat.objects.filter(
                sender=other_user_id,
                receiver=current_user,
                is_read=False
            ).count()
            unread_message_count[other_user_id] = unread_messages

        # Fetch user details for all users in the conversations
        users = User.objects.filter(id__in=user_ids).select_related("profile", "active_status")
        response_data = []

        for user in users:
            response_data.append({
                "id": user.id,
                "username": user.username,
                "full_name": user.get_full_name(),
                "email": getattr(user, "email", None),
                "profile_image": (
                    request.build_absolute_uri(user.profile.profile_image.url)
                    if hasattr(user, "profile") and user.profile.profile_image
                    else None
                ),
                "status": user.is_active,
                "conversation_id": user.id,  # Assuming each conversation is uniquely identified by user_id
                "last_message": last_message_map[user.id]["message"] if user.id in last_message_map else None,
                "last_message_time": last_message_map[user.id]["timestamp"] if user.id in last_message_map else None,
                "unread_messages": unread_message_count.get(user.id, 0),
                "current_status": (
                    user.active_status.get_status_display()
                    if hasattr(user, "active_status")
                    else "Offline"
                ),
            })

        return Response(response_data)
    
# APP_ID = config('APP_ID')
# APP_CERTIFICATE = config('APP_CERTIFICATE')
# AGORA_API_URL = "https://a41.chat.agora.io/411270423/1464694/chatrooms"
expire= 3600
class ChatRoomAPIView(APIView):

    def post(self, request):
        user = request.user
        name = request.data.get("name","")
        members = request.data.get("members", [])
        max_users = request.data.get("maxusers", 2)  # Default to 2 users

        if not members:
            return Response({"error": "Members are required"}, status=status.HTTP_400_BAD_REQUEST)

        description = "Personal Chat" if max_users == 2 else "Group Chat"

        # Validate members exist in the system
        try:
            member_users = User.objects.filter(username__in=members)
            print(member_users)
            if len(member_users) != len(members):
                return Response({"error": "Some members do not exist"}, status=status.HTTP_400_BAD_REQUEST)
        except User.DoesNotExist:
            return Response({"error": "Invalid members"}, status=status.HTTP_404_NOT_FOUND)

        # Generate Agora Token
        # app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, expire=3600)
        app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, expire=settings.AGORA_EXPIRE_TIME)
        cleaned_members = [User.objects.get(username=member).agora_username for member in members]
        cleaned_owner = user.agora_username

        # Agora API Payload
        payload = {
            "name": name,
            "description": description,
            "maxusers": max_users,
            "owner": cleaned_owner,
            "members": cleaned_members
        }

        # Call Agora API to create a chat room
        print(payload)
        response = requests.post(
            settings.AGORA_BASE_URL + "/chatrooms",
            json=payload,
            headers={"Content-Type": "application/json", "Authorization": f"Bearer {app_token}"}
        )
        print("Agora API Response Code:", response.status_code)
        print("Agora API Response:", response.text)
        if response.status_code == 200:
            agora_response = response.json()
            conversation_id = agora_response.get("data", {}).get("id")

            # Store conversation in the database
            conversation = ChatRoom.objects.create(
                conversation_id=conversation_id,
                name=name,
                description=description,
                max_users=max_users,
                owner=user
            )
            conversation.members.set(member_users)

            return Response({
                "conversation_id": conversation_id,
                "current_user_profile_image":None,
                "member_user_profile_image":None,
                "message": "Chat room created successfully"
            }, status=status.HTTP_201_CREATED)

        return Response({"error": "Failed to create chat room"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    

class ChatRoomListAPIView(APIView):

    def get(self, request):
        user = request.user
        print(user)
        # chat_rooms = ChatRoom.objects.filter(members=user, owner = user).order_by("-created_at")
        chat_rooms = ChatRoom.objects.filter(
            Q(members=user) | Q(owner=user)
        ).distinct().order_by("-created_at")

        group_chats = GroupChat.objects.filter(members = user).order_by("-created_at")
        # print(group_chats[0].agora_group_id)

        chat_room_data = []

        for chat_room in chat_rooms:
            members_data = []

            for member in chat_room.members.all():
                # Fetch last message content between the user and the member
                last_message = PersonalChat.objects.filter(
                    Q(sender=user, receiver=member) | Q(sender=member, receiver=user)
                ).order_by("-timestamp").first()
                # presence_status = getattr(member.active_status_chat_app, 'status', 3)

                decrypted_message = None
                last_message_time = None
                media_files_data = []
                if last_message:
                    # Decrypt message content if available
                    if last_message.content:
                        try:
                            decrypted_message = cipher.decrypt(last_message.content.encode()).decode()
                            last_message_time = last_message.timestamp
                        except Exception as e:
                            decrypted_message = "Error decrypting message"

                    # Extract media file details
                    for media in last_message.media_files.all():
                        media_files_data.append({
                            "media_type": media.media_type,
                            "file_name": media.file.name.split('/')[-1]
                            # "file_url": request.build_absolute_uri(media.file.url)  # Full URL to the file
                        })

                unread_message_count = PersonalChat.objects.filter(
                    sender=member,
                    receiver=user,
                    is_read=False
                ).count()

                members_data.append({
                    "id": member.id,
                    "username": member.username,
                    "email": member.mobile_or_email,
                    "full_name": member.full_name,
                    "profile_image": (
                        request.build_absolute_uri(member.profile.profile_image.url)
                        if hasattr(member, "profile") and member.profile.profile_image
                        else None
                    ),
                    "active_status": (
                        member.active_status_chat_app.get_status_display() 
                        if hasattr(member, "active_status_chat_app") else "Offline"
                    ),
                    # "presence_status": presence_status,  
                    "last_message": decrypted_message,
                    "last_message_time": last_message_time,
                    "unread_message_count":unread_message_count,
                    "media_files": media_files_data 
                })

            last_message = PersonalChat.objects.filter(
                Q(sender=chat_room.owner, receiver=user) | Q(sender=user, receiver=chat_room.owner)
            ).order_by("-timestamp").first()

            decrypted_message = None
            last_message_time = None
            if last_message:
                # Decrypt message content if available
                if last_message.content:
                    try:
                        decrypted_message = cipher.decrypt(last_message.content.encode()).decode()
                        last_message_time = last_message.timestamp
                    except Exception as e:
                        decrypted_message = "Error decrypting message"

                # Extract media file details
                for media in last_message.media_files.all():
                    media_files_data.append({
                        "media_type": media.media_type,
                        "file_name": media.file.name.split('/')[-1],
                        "file_url": request.build_absolute_uri(media.file.url)
                    })

            # Count unread messages for owner
            unread_message_count_owner = PersonalChat.objects.filter(
                sender=chat_room.owner,  # Messages sent by any member
                receiver__in=chat_room.members.all(),  # Received by owner
                is_read=False
            ).count()


            chat_room_data.append({
                "conversation_id": chat_room.conversation_id,
                "name": chat_room.name,
                "description": chat_room.description,
                "owner_or_member":  "owner" if chat_room.owner.id == user.id else "member",
                "owner": {
                    "id": chat_room.owner.id,
                    "username": chat_room.owner.username,
                    "email": chat_room.owner.mobile_or_email,
                    "full_name":chat_room.owner.full_name,
                    "profile_image": (
                    request.build_absolute_uri(chat_room.owner.profile.profile_image.url)
                    if hasattr(chat_room.owner, "profile") and chat_room.owner.profile.profile_image
                    else None
                ),
                "active_status": (
                        chat_room.owner.active_status_chat_app.get_status_display() 
                        if hasattr(chat_room.owner, "active_status_chat_app") else "Offline"
                    ),
                    "unread_message_count": unread_message_count_owner,
                    "last_message": decrypted_message,
                    "last_message_time": last_message_time,
                    "media_files": media_files_data 

                },
                "members": members_data,
                "max_users": chat_room.max_users,
                "created_at": chat_room.created_at
            })

            group_chat_data = []
            for group in group_chats:
                last_group_message = group.messages.order_by("-timestamp").first()
                decrypted_message = None
                media_files_data = []

                if last_group_message:
                    # If media files exist, prioritize them over text content
                    media_files = last_group_message.media_files.all()
                    if media_files.exists():
                        media_files_data = [
                            {
                                "media_type": media.media_type,
                                "file_name": media.file.name.split('/')[-1],
                                "file_url": request.build_absolute_uri(media.file.url),
                            }
                            for media in media_files
                        ]
                    elif last_group_message.content:
                        try:
                            decrypted_message = cipher.decrypt(last_group_message.content.encode()).decode()
                        except Exception as e:
                            decrypted_message = "Error decrypting message"
                group_chat_data.append({
                    "id": group.id,
                    "name": group.name,
                    "group_agora_id":group.agora_group_id,
                    "description": group.description,
                    "admin": group.admin.username,
                    "profile_image": group.profile_image.url if group.profile_image else None,
                    "last_message": media_files_data if media_files_data else decrypted_message,
                    # "last_message": last_group_message.content if last_group_message else None,
                    "last_message_time": last_group_message.timestamp if last_group_message else None,
                    "created_at": group.created_at
                })

            all_data = chat_room_data + group_chat_data
            all_data.sort(key=lambda x: x["created_at"], reverse=True)

        return Response({"chat_rooms": all_data}, status=status.HTTP_200_OK)
    
class ClearConversationAPI(APIView):
    def delete(self, request, other_user_id):
        user = request.user
        other_user = get_object_or_404(User, id=other_user_id)

        # Retrieve all personal chats between the two users
        chats = PersonalChat.objects.filter(
            (Q(sender=user) & Q(receiver=other_user)) |
            (Q(sender=other_user) & Q(receiver=user))
        )

        if not chats.exists():
            return Response(
                {"error": "No conversation found between the users."},
                status=status.HTTP_404_NOT_FOUND
            )

        # Mark the conversation as deleted for the requesting user
        for chat in chats:
            chat.deleted_for.add(user)

        # Prepare to delete the conversation from Agora
        agora_url = f"{settings.AGORA_BASE_URL}/users/{user.agora_username}/user_channel"
        payload = {
            "channel": other_user.agora_username,
            "type": "chat",
            "delete_roam": False
        }
        app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, expire=settings.AGORA_EXPIRE_TIME)
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {app_token}",
        }

        # Send the delete request to Agora
        # response = requests.delete(agora_url, json=payload, headers=headers)

        # if response.status_code != 200:
        #     # Log the error or handle it appropriately
        #     print("Agora API Error:", response.status_code, response.text)
        #     return Response(
        #         {"error": "Failed to delete conversation via Agora"},
        #         status=status.HTTP_500_INTERNAL_SERVER_ERROR
        #     )

        return Response(
            {"success": "Conversation deleted successfully."},
            status=status.HTTP_200_OK
        )
    

class SoftDeleteChatRoomAPI(APIView):
    def delete(self, request, chatroom_id):
        user = request.user
        chatroom = get_object_or_404(ChatRoom, id=chatroom_id)

        if user not in chatroom.members.all():
            return Response({"error": "You are not a member of this chat room."}, status=status.HTTP_403_FORBIDDEN)

        # Mark the chat room as deleted for the user
        chatroom.deleted_for.add(user)

        # Mark all messages in this chat room as deleted for this user
        PersonalChat.objects.filter(
            sender=user, receiver__in=chatroom.members.all()
        ).update(deleted_for=models.F('deleted_for') + [user])

        PersonalChat.objects.filter(
            receiver=user, sender__in=chatroom.members.all()
        ).update(deleted_for=models.F('deleted_for') + [user])

        return Response({"success": "Chat room and all its messages deleted for you."}, status=status.HTTP_200_OK)

from django.http import JsonResponse
from django.shortcuts import get_object_or_404
# from agora_token_builder import RtcTokenBuilder  
from src.RtcTokenBuilder2 import *

class Calltoken(APIView):
    def post(self, request):
        # Extract receiver_id from request
        receiver_id = request.data.get("receiver_id")
        if not receiver_id:
            return JsonResponse({"error": "receiver_id is required"}, status=400)

        # Fetch sender and receiver
        sender = request.user
        receiver = get_object_or_404(User, id=receiver_id)

        # Ensure both users have agora_username
        if not sender.agora_username or not receiver.agora_username:
            return JsonResponse({"error": "Both users must have an agora_username"}, status=400)

        # Generate a unique channel_name using sender and receiver usernames
        channel_name = f"{sender.agora_username}_{receiver.agora_username}"
        app_id = os.environ.get("APP_ID")
        # Need to set environment variable AGORA_APP_CERTIFICATE
        app_certificate = os.environ.get("APP_CERTIFICATE")

        CallSession.objects.filter(
            (Q(sender=sender, receiver=receiver) | Q(sender=receiver, receiver=sender))
        ).delete()

        '''if call_session:
            return JsonResponse({
                "status":"Already_exist",
                "token": call_session.token,
                "channel_name": call_session.channel_name,
                "uid": call_session.uid,
                "app_id": app_id,
            })'''

        uid = int(receiver_id)
        print(uid)
        # uid = sender.id
        # channel_name = "7d72365eb983485397e3e3f9d460bdda"
        # uid = 2882341273
        # account = "2882341273"

        # Generate Agora token
        
        # print(app_id)
        # print(app_certificate)

        # role = 2
        token_expire = 315360000  # Token expiry time (1 hour)
        privilege_expire = 600

        # token = RtcTokenBuilder.build_token_with_uid(
        #     app_id, app_certificate, channel_name, uid, Role_Subscriber,
        #                                          token_expiration_in_seconds, privilege_expiration_in_seconds
        # )
        #role = Role_Subscriber 
        '''token = RtcTokenBuilder.build_token_with_uid(app_id, app_certificate, channel_name, uid, role,
                                                 token_expire, privilege_expire)'''
        role = Role_Publisher 
        # token = RtcTokenBuilder.build_token_with_uid(app_id, app_certificate, channel_name, uid, role,
        #                                          token_expire, privilege_expire)
        sender_token = RtcTokenBuilder.build_token_with_uid(app_id, app_certificate, channel_name, sender.id, role,
                                                             token_expire, privilege_expire)
        receiver_token = RtcTokenBuilder.build_token_with_uid(app_id, app_certificate, channel_name, receiver.id, role,
                                                               token_expire, privilege_expire)

        # token = RtcTokenBuilder.build_token_with_uid(app_id, app_certificate, channel_name, uid, Role_Publisher,
        #                                          token_expiration_in_seconds, privilege_expiration_in_seconds)                                        

        call_session = CallSession.objects.create(
            sender=sender,
            receiver=receiver,
            channel_name=channel_name,
            token=sender_token,
            uid=uid
        )
        # Return token, channel_name, and uid to the frontend
        return JsonResponse({
            "sender": {
                "id": sender.id,
                "token": sender_token,
            },
            "receiver": {
                "id": receiver.id,
                "token": receiver_token,
            },
            "channel_name": channel_name,
            "uid": uid,
            "app_id": app_id,
        })
    
class GroupMessagesView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, group_id):
        group = get_object_or_404(GroupChat, id=group_id)
        messages = group.messages.exclude(deleted_for=request.user).order_by('timestamp')
        page = request.GET.get('page', 1)  # Default to page 1
        page_size = request.GET.get('page_size', 10)  # Default page size = 10
        paginator = Paginator(messages, page_size)
        paginated_messages = paginator.get_page(page)

        messages_data = []
        for message in paginated_messages:
            try:
                decrypted_content = cipher.decrypt(message.content.encode()).decode() if message.content else None
            except Exception:
                decrypted_content = "Error decrypting message"

            sender_status = getattr(message.sender, 'active_status_chat_app', None)
            presence_status = sender_status.status if sender_status else 3  # Default to 'Offline'

            messages_data.append({
                "id": message.id,
                "sender_id": message.sender.id,
                "sender_name": message.sender.full_name,
                "sender_profile_image": request.build_absolute_uri(message.sender.profile.profile_image.url) if hasattr(message.sender, 'profile') and message.sender.profile.profile_image else None,
                "content": decrypted_content,
                "timestamp": message.timestamp,
                # "media_type":message.media_files.media_type,
                # "media_files": [request.build_absolute_uri(media.file.url) for media in message.media_files.all()],
                "media_files": [
                    {
                        "file_url": request.build_absolute_uri(media.file.url),
                        "media_type": media.media_type
                    }
                    for media in message.media_files.all()
                ],
                "user_type": "sender" if message.sender.id == request.user.id else "receiver",
                "presence_status": presence_status,
            })
        response_data = {
            "group_id": group.id,
            "group_name": group.name,
            "group_profile_image": request.build_absolute_uri(group.profile_image.url) if group.profile_image else None,
            "messages": messages_data,
            "current_page": paginated_messages.number,
            "total_pages": paginator.num_pages,
            "total_items": paginator.count,
            "has_next": paginated_messages.has_next(),
            "has_previous": paginated_messages.has_previous(),
        }

        return Response(response_data)
    
class GroupMembersListView(APIView):
    permission_classes = [IsAuthenticated]  # Only authenticated users can access

    def get(self, request, group_id):
        # Fetch the group or return 404 if not found
        group = get_object_or_404(GroupChat, id=group_id)

        # Get all members including the admin
        members = group.members.all()

        # Serialize member data
        members_data = [{
            "id": member.id,
            "username":member.username,
            "full_name": member.full_name,
            "profile_image": request.build_absolute_uri(member.profile.profile_image.url) if hasattr(member, 'profile') and member.profile.profile_image else None,
            "is_admin": member == group.admin
        } for member in members]

        # Prepare final response
        response_data = {
            "group_id": group.id,
            "group_name": group.name,
            "group_profile_image": request.build_absolute_uri(group.profile_image.url) if group.profile_image else None,
            "members": members_data
        }

        return Response(response_data)
    
    
class RemoveGroupMemberView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, group_id, user_id):
        """
        API to allow ONLY an admin to remove a user.
        If the admin exits, it triggers an API call to Agora to remove them as admin.
        """
        # Fetch group or return 404
        group = get_object_or_404(GroupChat, id=group_id)
        user_to_remove = get_object_or_404(User, id=user_id)

        # Check if the requesting user is the group admin
        # if request.user != group.admin:
        #     return Response({"error": "Only the group admin can remove members."},
        #                     status=status.HTTP_403_FORBIDDEN)

        is_admin_exiting = user_id == group.admin.id
        print(user_id)
        print(group.admin.id)
        print(is_admin_exiting)
        is_admin_removing = request.user == group.admin and request.user != user_to_remove
        is_normal_user_exiting = request.user == user_to_remove and request.user != group.admin

        if not (is_admin_removing or is_normal_user_exiting or is_admin_exiting):
            return Response({"error": "You do not have permission to remove this user."},
                            status=status.HTTP_403_FORBIDDEN)


        group.members.remove(user_to_remove)

        app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, settings.AGORA_EXPIRE_TIME)

        # If the admin exits, call the Agora API to remove them as admin
        print(user_to_remove.agora_username)
        if is_admin_exiting:
            # left_members = GroupChat.objects.filter(group = )
            # delete_group = f"{settings.AGORA_BASE_URL}/chatgroups/{group.agora_group_id}"
            admin_ownership_transfer = f"{settings.AGORA_BASE_URL}/chatgroups/{group.agora_group_id}"
            agora_url = f"{settings.AGORA_BASE_URL}/chatgroups/{group.agora_group_id}/users/{user_to_remove.agora_username}"
        else:
            agora_url = f"{settings.AGORA_BASE_URL}/chatgroups/{group.agora_group_id}/users/{user_to_remove.agora_username}"

        agora_headers = {
            "Authorization": f"Bearer {app_token}",
            "Content-Type": "application/json"
        }

        if is_admin_exiting:
            print("admin exiting")
            remaining_members = group.members.exclude(id=user_to_remove.id)
            if remaining_members.exists():
                new_admin = remaining_members.first()
                group.admin = new_admin
                print(new_admin.agora_username)
                payload = { "newowner": new_admin.agora_username }
                ownership_change_response = requests.put(admin_ownership_transfer, json= payload, headers=agora_headers )
                if ownership_change_response.status_code not in [200, 204]:
                    return Response({"error": "Failed to change ownership of Agora chat group", "details": response.json()},
                                    status=response.status_code)
                group.save()
                message = f"Admin {user_to_remove.full_name} exited. New admin is {new_admin.full_name}."
            else:
                delete_group_url = f"{settings.AGORA_BASE_URL}/chatgroups/{group.agora_group_id}"
                delete_group_response = requests.delete(delete_group_url, headers=agora_headers)

                if delete_group_response.status_code not in [200, 204]:
                    return Response({"error": "Failed to delete Agora chat group", "details": delete_group_response.json()},
                                    status=delete_group_response.status_code)

                group.delete()
                return Response({"message": f"Group '{group.name}' has been deleted as the last admin exited."},
                                status=status.HTTP_200_OK)
                # message = f"Admin {user_to_remove.full_name} exited. The group has no members left."
        elif is_normal_user_exiting:
            message = f"{user_to_remove.full_name} has exited the group."

        elif is_admin_removing:
            message = f"{user_to_remove.full_name} has been removed from the group by admin."

        

        response = requests.delete(agora_url, headers=agora_headers)

        if response.status_code not in [200, 204]:
            return Response({"error": "Failed to remove user from Agora chat group", "details": response.json()},
                            status=response.status_code)

        return Response({
            "message": message,
            "group_id": group_id,
            "removed_user": {
                "id": user_to_remove.id,
                "full_name": user_to_remove.full_name,
                "agora_username": user_to_remove.agora_username
            },
            "is_admin_exit": is_admin_exiting
        }, status=status.HTTP_200_OK)
    
class ReportGroupView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, group_id):
        group = get_object_or_404(GroupChat, id=group_id)

        if GroupReport.objects.filter(reporter=request.user, group=group).exists():
            return Response({
                "success": False,
                "message": "You have already reported this group."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Create a report entry
        GroupReport.objects.create(reporter=request.user, group=group)

        # Send notification to the group admin
        Notification.objects.create(
            user=group.admin,
            action_type="reported",
            message=f"Your group '{group.name}' has been reported by {request.user.username}."
        )

        return Response({
            "success": True,
            "message": "Group reported successfully. The group owner has been notified."
        }, status=status.HTTP_201_CREATED)
    
class DeleteMessageAPI(APIView):
    permission_classes = [IsAuthenticated]

    def delete(self, request, message_id):
        user = request.user
        message = get_object_or_404(PersonalChat, id=message_id)
        if message.sender != user and message.receiver != user:
            return Response({"error": "You do not have permission to delete this message."}, status=status.HTTP_403_FORBIDDEN)

        message.deleted_for.add(user)
        message.save()

        return Response({"message": "Message deleted successfully on your side."}, status=status.HTTP_200_OK)
    
class DeleteGroupMessageAPI(APIView):
    """
    API to delete a single message from a group chat only for the requesting user.
    """
    permission_classes = [IsAuthenticated]

    def delete(self, request, message_id):
        user = request.user
        message = get_object_or_404(GroupMessage, id=message_id)

        # Check if the user is a member of the group
        if user not in message.group.members.all():
            return Response({"error": "You do not have permission to delete this message."}, status=status.HTTP_403_FORBIDDEN)

        # Mark the message as deleted for the user
        message.deleted_for.add(user)
        message.save()

        return Response({"message": "Message deleted successfully on your side."}, status=status.HTTP_200_OK)


class DeleteGroupConversationAPI(APIView):
    """
    API to delete an entire group chat conversation only for the requesting user.
    """
    permission_classes = [IsAuthenticated]

    def delete(self, request, group_id):
        user = request.user
        group = get_object_or_404(GroupChat, id=group_id)

        # Check if the user is a member of the group
        if user not in group.members.all():
            return Response({"error": "You are not a member of this group."}, status=status.HTTP_403_FORBIDDEN)

        # Get all messages in the group and mark them as deleted for the user
        messages = group.messages.all()
        for message in messages:
            message.deleted_for.add(user)
            message.save()

        return Response({"message": "Conversation deleted successfully on your side."}, status=status.HTTP_200_OK)
      
from firebase_admin import messaging
def send_push_notification(fcm_token, title, body, data_payload):
    """Send push notification using Firebase Cloud Messaging (FCM)."""
    if not fcm_token:
        return {"status": "error", "message": "FCM token is missing"}

    message = messaging.Message(
        notification=messaging.Notification(title=title, body=body),
        data=data_payload,
        token=fcm_token,
    )

    try:
        response = messaging.send(message)
        return {"status": "success", "message": "Notification sent", "response": response}
    except Exception as e:
        return {"status": "error", "message": str(e)}
    
class PushNotificationAPIView(APIView):
    """API to send push notifications for messages, voice calls, or video calls."""

    def post(self, request):
        recipient_id = request.data.get("user_id")
        notification_type = request.data.get("type")
        call_type = request.data.get("call_type")
        message = request.data.get("message", "")
        channel_name = request.data.get("channel_name", "")
        token = request.data.get("token", "")
        username = request.data.get("username", "")  # Sender username
        profile_image = request.data.get("profile_image", "")  # Sender profile image
        app_id = os.environ.get("AGORA_APP_ID")

        # Fetch recipient
        try:
            recipient = User.objects.get(id=recipient_id)
        except User.DoesNotExist:
            return Response({"status": "error", "message": "Recipient not found"}, status=status.HTTP_404_NOT_FOUND)

        # Check if recipient has an FCM token
        fcm_token = getattr(recipient, "fcm_token", None)
        print(fcm_token)
        if not fcm_token:
            return Response({"status": "error", "message": "Recipient has no FCM token"}, status=status.HTTP_400_BAD_REQUEST)

        # Notification title & body
        title = "New Notification"
        body = f"You have a new {notification_type.replace('_', ' ')} from {username}"

        # Data payload
        data_payload = {
            "user_id": str(recipient.id),
            "type": notification_type,  # "message" or "call"
            "call_type": call_type if notification_type in ["video_call", "audio_call"] else "message",
            "message": message,
            "channel_name": channel_name,
            "token": token,
            "username": username,
            "profile_image": profile_image,
          	"app_id": app_id,
        }
        print(data_payload)

        # Send push notification
        response = send_push_notification(fcm_token, title, body, data_payload)
        return Response(response, status=status.HTTP_200_OK if response["status"] == "success" else status.HTTP_400_BAD_REQUEST)